/*
 * Copyright CINES Centre Informatique National de l'Enseignement Supérieur, 2017
 * Tout droit d'utilisation strictement soumis à l'approbation du CINES
 */
package fr.gouv.vitamui.pastis.model.jaxb;

import fr.gouv.vitamui.pastis.model.ElementProperties;
import fr.gouv.vitamui.pastis.util.RNGConstants;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.xml.bind.annotation.*;
import java.util.ArrayList;
import java.util.List;


@XmlRootElement
public class BaliseXML{


	public BaliseXML() {
		super();
	}

	String name;
	String dataType;
	String cardinality;
	String groupOrChoice;
	BaliseXML parent;
	ValueXML valueXML;

	private static final Logger LOGGER = LoggerFactory.getLogger(BaliseXML.class);

	public static BaliseXML baliseXMLStatic;

	List<BaliseXML> children = new ArrayList<>();

	@XmlAttribute
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	@XmlAttribute (name="type")
	public String getDataType() {
		return dataType;
	}

	public void setDataType(String dataType) {
		this.dataType = dataType;
	}


	//@XmlTransient to avoid circular loop parent <-> child
	@XmlTransient
	public BaliseXML getParent() {
		return parent;
	}

	public void setParent(BaliseXML parent) {
		this.parent = parent;
	}

	// XmlAnyElement pour etre le plus generique
	@XmlAnyElement
	public List<BaliseXML> getChildren() {
		return children;
	}

	public void setChildren(List<BaliseXML> children) {
		this.children = children;
	}

	public String getCardinality() {
		return cardinality;
	}

	public String setCardinality(RNGConstants.Cardinality cardinality) {
		return this.cardinality = cardinality.getLabel();
	}

	public String getGroupOrChoice() {
		return groupOrChoice;
	}

	@XmlElement(name="rng:value")
	public ValueXML getValueXML() {
		return valueXML;
	}

	public void setValueXML(ValueXML valueXML) {
		this.valueXML = valueXML;
	}

	public void setGroupOrChoice(RNGConstants.GroupOrChoice groupOrChoice) {
		this.groupOrChoice = groupOrChoice.getLabel();
	}


	@Override
	public String toString() {
		// TODO Auto-generated method stub
		return this.name;
	}

	/**
	 *
	 * @param node node représentant l'arbre ElementProperties correspondant au json reçu du front
	 * @param profondeur profondeur du noeud utile pour le front angular
	 * @param parentNode noeud parent utilisé dans la récursivité pour lié parent & children
	 */
	public static void buildBaliseXMLTree(ElementProperties node, int profondeur, BaliseXML parentNode ){

		ValueXML valueRNG = null;
		DataXML dataRNG = null;
		BaliseXML cardinalityRNG = null;
		BaliseXML elementOrAttributeRNG = null;
		AnnotationXML annotationXML = null;
		DocumentationXML documentationXML = null;
		BaliseXML groupOrChoice = null;

		// If the node has a value
		if(null != node.getValue() && !node.getValue().equals("undefined")) {
			valueRNG = new ValueXML();
			valueRNG.setValue(node.getValue());
		}

		// When a value is declared in a profile element, the <rng:data> tag must be suppressed
		// to assure that the generated profile is successfully imported by VITAM
		if(null != node.getValueOrData() && !node.getValueOrData().equals("undefined") && node.getValue() == null) {
			if(node.getValueOrData().equals("data")) {
				dataRNG = new DataXML();
			}
		}
		// Sets the type of data (if value or data)
		if(null != node.getDataType() && !node.getDataType().equals("undefined")) {
			if(null != valueRNG) {
				valueRNG.setDataType(node.getDataType());
			}else
				if(null != dataRNG) {
				dataRNG.setDataType(node.getDataType());
			}
		}
		// Set annotation and documentation tags (if exists)
		if(null != node.getDocumentation() ) {
			annotationXML = new AnnotationXML();
			documentationXML = new DocumentationXML();
			documentationXML.setDocumentation(node.getDocumentation());
			annotationXML.setDocumentationXML(documentationXML);
		}

		if(null != node.getType() && !node.getType().equals("undefined")) {
			if(node.getType().equals("element")) {
				elementOrAttributeRNG = new ElementXML();
			} else if(node.getType().equals("attribute")) {
				elementOrAttributeRNG = new AttributeXML();
			}
			if(null != node.getName() && !node.getName().equals("undefined")) {
				elementOrAttributeRNG.setName(node.getName());
			}
		}

		if(null != documentationXML) {
			elementOrAttributeRNG.getChildren().add(annotationXML);
			annotationXML.setParent(elementOrAttributeRNG);
		}

		// Check node's and its children's cardinality
		if(node.getCardinality() != null ) {
			if(node.getCardinality().equals(RNGConstants.Cardinality.zeroOrMore.getLabel())) {
				cardinalityRNG = new ZeroOrMoreXML();
				if(elementOrAttributeRNG != null) {
					elementOrAttributeRNG.setCardinality(RNGConstants.Cardinality.zeroOrMore);
				}
			} else if(node.getCardinality().equals(RNGConstants.Cardinality.oneOrMore.getLabel())) {
				cardinalityRNG = new OneOrMoreXML();
				if(elementOrAttributeRNG != null) {
					elementOrAttributeRNG.setCardinality(RNGConstants.Cardinality.oneOrMore);
				}
			} else if(node.getCardinality().equals(RNGConstants.Cardinality.optional.getLabel())) {
				cardinalityRNG = new OptionalXML();
				if(elementOrAttributeRNG != null) {
					elementOrAttributeRNG.setCardinality(RNGConstants.Cardinality.optional);
				}
			}
		}

		BaliseXML currentXmlTag = null;

		// 1. Check if it is an element
		if(null != elementOrAttributeRNG) {

			LOGGER.info("Parsing " + elementOrAttributeRNG.getName());
			// 1.1 Check if the element has cardinality
			if( null != cardinalityRNG ) {
				cardinalityRNG.getChildren().add(elementOrAttributeRNG);
				elementOrAttributeRNG.setParent(cardinalityRNG);
				currentXmlTag = cardinalityRNG;
			}

			else {
				currentXmlTag = elementOrAttributeRNG;
				//1.2. Check if it's the first grammarnode (Archive transfer)
				if(parentNode == null) {
					GrammarXML grammar = new GrammarXML();
					StartXML start = new StartXML();
					start.setParent(grammar);
					grammar.getChildren().add(start);
					currentXmlTag = grammar;
				}
			}

			// 2. Check data tag
            if(null != dataRNG) {
            	if (!currentXmlTag.getChildren().isEmpty() &&
            			(currentXmlTag.getChildren().get(0) instanceof ElementXML
            					||currentXmlTag.getChildren().get(0) instanceof AttributeXML)){
                	currentXmlTag.getChildren().get(0).getChildren().add(dataRNG);
                    dataRNG.setParent(currentXmlTag);
                }else {
                	currentXmlTag.getChildren().add(dataRNG);
                    dataRNG.setParent(currentXmlTag);
                }
			}
			// 3. Check value tag
			if(null != valueRNG) {
				// If Children is empty
				if(currentXmlTag.getChildren().isEmpty() ) {
					if (currentXmlTag instanceof ElementXML) {
						((ElementXML)currentXmlTag).setValueXML(valueRNG);
						valueRNG.setParent(currentXmlTag);

					} else if (currentXmlTag instanceof AttributeXML) {
						((AttributeXML)currentXmlTag).setValueXML(valueRNG);
						valueRNG.setParent(currentXmlTag);
					}
					// If children is Element or Attribute, set  accordingly
				} else if (currentXmlTag instanceof ElementXML) {
					((ElementXML) currentXmlTag).setValueXML(valueRNG);
					valueRNG.setParent(currentXmlTag);

				} else if (currentXmlTag instanceof AttributeXML) {
					((AttributeXML) currentXmlTag).setValueXML(valueRNG);
					valueRNG.setParent(currentXmlTag);
				}
				else {
					// Set the value to an simple element
					elementOrAttributeRNG.setValueXML(valueRNG);
					valueRNG.setParent(elementOrAttributeRNG);
				}
            }
		}

		if(null != currentXmlTag) {

			if(null != parentNode) {
				BaliseXML optionalWithChildren = parentNode.getChildren()
						.stream().filter(cardinality-> cardinality instanceof OptionalXML
								|| cardinality instanceof ZeroOrMoreXML
								|| cardinality instanceof OneOrMoreXML )
						.findAny()
						.orElse(null);

				Boolean optionalHasAlreadyCurrentTag = optionalWithChildren == null
						? false : optionalWithChildren.children.contains(currentXmlTag);

				if (!optionalHasAlreadyCurrentTag) {
					currentXmlTag.setParent(parentNode);
					parentNode.getChildren().add(currentXmlTag);
				}


			}else {
				baliseXMLStatic = currentXmlTag;
			}
		}

		if (currentXmlTag instanceof GrammarXML) {
			buildBaliseXMLTree( node, profondeur + 1, currentXmlTag.getChildren().get(0));
		}
		else {
			for( ElementProperties next : node.getChildren() ) {
				if (currentXmlTag instanceof OptionalXML || currentXmlTag instanceof OneOrMoreXML
					|| currentXmlTag instanceof ZeroOrMoreXML)  {
					buildBaliseXMLTree( next, profondeur + 1, currentXmlTag.getChildren().get(0));
				} else {
					buildBaliseXMLTree( next, profondeur + 1, currentXmlTag );
				}
			}
		}

	}




}
